import { Injectable } from '@nestjs/common';
import { PubSub } from 'graphql-subscriptions';
import { RedisService } from 'nestjs-redis';
import { omit } from 'ramda';
import { PipelineTaskLogMessage } from './models/pipeline-task-log-message.module';
import { PipelineTask } from './pipeline-task.entity';

const LOG_TIMEOUT_SECONDS = 10_000;

@Injectable()
export class PipelineTaskLogsService {
  constructor(private readonly redisService: RedisService) {}

  pubSub = new PubSub();

  get redis() {
    return this.redisService.getClient();
  }

  getKeys(task: PipelineTask) {
    return `ptl:${task.id}`;
  }

  async recordLog(log: PipelineTaskLogMessage) {
    const logDto = omit(['task'], log);
    await Promise.all([
      this.pubSub.publish(this.getKeys(log.task), logDto),
      this.redis
        .expire(this.getKeys(log.task), LOG_TIMEOUT_SECONDS)
        .then(() =>
          this.redis.rpush(this.getKeys(log.task), JSON.stringify(logDto)),
        ),
    ]);
  }

  async readLogs(task: PipelineTask): Promise<PipelineTaskLogMessage[]> {
    return await this.redis.lrange(this.getKeys(task), 0, -1).then((items) =>
      items.map((item) => {
        const log = JSON.parse(item) as PipelineTaskLogMessage;
        log.task = task;
        log.time = new Date(log.time);
        return log;
      }),
    );
  }

  watchLogs(task: PipelineTask) {
    return this.pubSub.asyncIterator(this.getKeys(task));
  }
}
